declare const React: any;
declare const Eui: Record<string, (...args: any[]) => any>;

const {
  EuiMarkdownFormat,
  EuiButton,
  EuiCallOut,
  EuiHorizontalRule,
  EuiLoadingSpinner,
  getDefaultEuiMarkdownParsingPlugins,
} = Eui;

const markdownPlugins = getDefaultEuiMarkdownParsingPlugins();
for (const plugin of markdownPlugins) {
  if (plugin[1]?.emoticon) {
    plugin[1].emoticon = false;
  }
}

async function main(): Promise<void> {
  await currentVisualization.openFlyout({
    titleText: 'Graph Report',
    Element: <FlyoutBody/>,
    size: 's'
  });
}

function FlyoutBody(): React.ReactElement {
  const [content, setContent] = React.useState('');
  const [isLoading, setIsLoading] = React.useState(true);
  const [error, setError] = React.useState();

  React.useEffect(() => {
    async function generateReport(): Promise<void> {
      setIsLoading(true);
      try {
        const [graph, selectedNodes] = await Promise.all([
          currentVisualization.getGraphModel(),
          currentVisualization.selection()
        ]);
        const selectedNodeIds = selectedNodes.map(node => node.id);
        const { content } = await sirenapi.extensions.GenerativeAi.generateGraphReport(graph, { focusNodes: selectedNodeIds });
        setContent(content);
      } catch (e: any) {
        setError(e.cause?.response?.data?.message || e.message);
      } finally {
        setIsLoading(false);
      }
    }
    void generateReport();
  }, [setContent, setError, setIsLoading]);

  if (error) {
    return <EuiCallOut title="Something went wrong querying the LLM" color="danger" iconType="alert">{error}</EuiCallOut>;
  }

  if (isLoading) {
    return (
      <div
        style={{ display: 'flex', flexDirection: 'column', justifyContent: 'center', margin: '32px 0', alignItems: 'center', gap: '16px' }}
      >
        <h2>Preparing your report...</h2>
        <EuiLoadingSpinner size="xl"/>
      </div>
    );
  }

  return (
    <div>
      <EuiCallOut
        size="s"
        title="This report was generated using a Generative AI, please validate the correctness of the report before use."
        iconType="help"
        style={{ marginBottom: '16px' }}
        data-test-subj="generative-ai-warning"
      />
      <div data-test-subj="generated-content">
        <EuiMarkdownFormat parsingPluginList={markdownPlugins}>{content}</EuiMarkdownFormat>
      </div>
      <EuiHorizontalRule size="full" margin="s"/>
      <EuiButton
        fill
        onClick={() => downloadReport(content)}
        iconType="download"
        data-test-subj="download-report"
      >
        Download Report
      </EuiButton>
    </div>
  );
}

async function downloadReport(markdown: string): Promise<void> {
  const reportBlob = await sirenapi.Reporting.markdownToDocxBlob(markdown);
  await sirenapi.Reporting.downloadBlob(reportBlob, 'Siren-Graph-Report.docx');
}

main()
  .catch(e => {
    console.error(e);
    sirenapi.notify.error(new Error(`An error occurred generating a report: ${e.message}`, { cause: e }));
  });
